; Jump to start
    ; set startstkp = sp for exit()
    ld  hl, 0
    add hl, sp
    ld  d, h
    ld  e, l
    ld  hl, startstkp
    ld  (hl), d
    inc  hl
    ld  (hl), e
    JP main
startstkp:
.dw 00, 00;
;	Run time start off for Small C.
;
;*****************************************************
;						     *
;	runtime	library	for small C compiler	     *
;						     *
;	c.s - runtime routine for basic C code	     *
;						     *
;		Ron Cain			     *
;						     *
;*****************************************************
;
; fetch char from (HL) and sign extend into HL
_gchar: ld      a,(hl)
_sxt:   ld      l,a
	rlca
	sbc	a,a
	ld	h,a
	ret
; fetch int from (HL)
_gint:  ld      a,(hl)
	inc	hl
	ld	h,(hl)
	ld	l,a
	ret
; store char from HL into (DE)
_pchar: ld      a,l
	ld	(de),a
	ret
; store int from HL into (DE)
_pint:  ld      a,l
	ld	(de),a
	inc	de
	ld	a,h
	ld	(de),a
	ret
; "or" HL and DE into HL
_or:    ld      a,l
	or	e
	ld	l,a
	ld	a,h
	or	d
	ld	h,a
	ret
; "xor" HL and DE into HL
_xor:   ld      a,l
	xor	e
	ld	l,a
	ld	a,h
	xor	d
	ld	h,a
	ret
; "and" HL and DE into HL
_and:   ld      a,l
	and	e
	ld	l,a
	ld	a,h
	and	d
	ld	h,a
	ret
;
;______logical operations: HL set to 0 (false) or 1 (true)
;
; DE == HL
_eq:    call    _cmp
        ret     z
	dec	hl
	ret
; DE != HL
_ne:    call    _cmp
	ret	nz
	dec	hl
	ret
; DE > HL [signed]
_gt:    ex      de,hl
        call    _cmp
	ret	c
	dec	hl
	ret
; DE <= HL [signed]
_le:    call    _cmp
	ret	z
	ret	c
	dec	hl
	ret
; DE >= HL [signed]
_ge:    call    _cmp
	ret	nc
	dec	hl
	ret
; DE < HL [signed]
_lt:    call    _cmp
	ret	c
	dec	hl
	ret
; DE >= HL [unsigned]
_uge:   call    _ucmp
	ret	nc
	dec	hl
	ret
; DE < HL [unsigned]
_ult:   call    _ucmp
	ret	c
	dec	hl
	ret
; DE > HL [unsigned]
_ugt:   ex      de,hl
        call    _ucmp
	ret	c
	dec	hl
	ret
; DE <= HL [unsigned]
_ule:   call    _ucmp
	ret	z
	ret	c
	dec	hl
	ret
; signed compare of DE and HL
;   carry is sign of difference [set => DE < HL]
;   zero is zero/non-zero
_cmp:   ld      a,e
	sub	l
	ld	e,a
	ld	a,d
	sbc	a,h
        ld      hl,1           ;preset true
        jp      m,_cmp1
        or      e              ;resets carry
	ret
_cmp1:  or      e
	scf
	ret
; unsigned compare of DE and HL
;   carry is sign of difference [set => DE < HL]
;   zero is zero/non-zero
_ucmp:  ld      a,d
	cp	h
        jr      nz,_ucmp1
	ld	a,e
	cp	l
_ucmp1: ld      hl,1           ;preset true
	ret
; shift DE right arithmetically by HL, move to HL
_asr:   ex      de,hl
_asr1:  dec     e
	ret	m
	ld	a,h
	rla
	ld	a,h
	rra
	ld	h,a
	ld	a,l
	rra
	ld	l,a
        jr      _asr1
; shift DE left arithmetically by HL, move to HL
_asl:   ex      de,hl
_asl1:  dec     e
	ret	m
	add	hl,hl
        jr      _asl1
; HL = DE - HL
_sub:   ld      a,e
	sub	l
	ld	l,a
	ld	a,d
	sbc	a,h
	ld	h,a
	ret
; HL = -HL
_neg:   call    _com
	inc	hl
	ret
; HL = ~HL
_com:   ld      a,h
	cpl
	ld	h,a
	ld	a,l
	cpl
	ld	l,a
	ret
; HL = !HL
_lneg:  ld      a,h
	or	l
        jr      z,_lneg1
        ld      hl,0
	ret
_lneg1: inc     hl
	ret
; HL = !!HL
_bool:  call    _lneg
        jr      _lneg
;
; HL = DE * HL [signed]
_mul:   ld      b,h
	ld	c,l
        ld      hl,0
_mul1:  ld      a,c
	rrca
        jr      nc,_mul2
	add	hl,de
_mul2:  xor     a
	ld	a,b
	rra
	ld	b,a
	ld	a,c
	rra
	ld	c,a
	or	b
	ret	z
	xor	a
	ld	a,e
	rla
	ld	e,a
	ld	a,d
	rla
	ld	d,a
	or	e
	ret	z
        jr      _mul1
; HL = DE / HL, DE = DE % HL
_div:   ld      b,h
	ld	c,l
	ld	a,d
	xor	b
	push	af
	ld	a,d
	or	a
        call    m,_deneg
	ld	a,b
	or	a
        call    m,_bcneg
        ld      a,16
	push	af
	ex	de,hl
        ld      de,0
_div1:  add     hl,hl
        call    _rdel
        jr      z,_div2
        call    _cmpbd
        jp      m,_div2
	ld	a,l
        or      1
	ld	l,a
	ld	a,e
	sub	c
	ld	e,a
	ld	a,d
	sbc	a,b
	ld	d,a
_div2:  pop     af
	dec	a
        jr      z,_div3
	push	af
        jr      _div1
_div3:  pop     af
	ret	p
        call    _deneg
	ex	de,hl
        call    _deneg
	ex	de,hl
	ret
; {DE = -DE}
_deneg: ld      a,d
	cpl
	ld	d,a
	ld	a,e
	cpl
	ld	e,a
	inc	de
	ret
; {BC = -BC}
_bcneg: ld      a,b
	cpl
	ld	b,a
	ld	a,c
	cpl
	ld	c,a
	inc	bc
	ret
; {DE <r<r 1}
_rdel:  ld      a,e
	rla
	ld	e,a
	ld	a,d
	rla
	ld	d,a
	or	e
	ret
; {BC : DE}
_cmpbd: ld      a,e
	sub	c
	ld	a,d
	sbc	a,b
	ret
; case jump
_case:  ex      de,hl                   ;switch value to DE
	pop	hl		;get table address
_case1: call    _case4          ;get case value
	ld	a,e
        cp      c               ;equal to switch value _
        jr      nz,_case2       ;no
	ld	a,d
        cp      b               ;equal to switch value _
        jr      nz,_case2       ;no
        call    _case4          ;get case label
        jr      z,_case3        ;end of table, go to default
	push	bc
	ret			;case jump
_case2: call    _case4          ;get case label
        jr      nz,_case1       ;next case
_case3: dec     hl
	dec	hl
	dec	hl
	ld	d,(hl)
	dec	hl
	ld	e,(hl)
	ex	de,hl
	jp	(hl)		;default jump
_case4: ld      c,(hl)
	inc	hl
	ld	b,(hl)
	inc	hl
	ld	a,c
	or	b
	ret
