/* TI-86 Definitons File */

#ifndef __TILOADED__
#define __TILOADED__ 1

/* Macros

Macro Name      Description
----------------------------------------------------------------------------
clrscrn         Clears The Text form the Screen
clrlcd          Clears the hole LCD screen
indicatoroff    This turns the annoying run indicator off
indicatoron     This turns the annoying run indicator on
displaygraph    This funtion display whatever grafing equations are stored
                in the calculator.
nl              This funtion makes a new line on the screen like a \n in
                Most C programing

*/

#define clrscrn         asm("\tcall\t_clrScrn\n")
#define clrlcd          asm("\tcall\t_clrLCD\n")

#define indicatoroff    asm("\tcall\t_runindicoff\n")
#define indicatoron     asm("\tcall\t_runindicon\n")

#define displaygraph    asm("\tcall\t_PDspGrph\n");

#define nl              asm("\tcall\t_newline\n");

/* Pixel Defiations

These are used when calling fuctions like line, pixel, and Box

PT_OFF          Turns the pixel/pixels off
PT_ON           Turns the pixel/pixels on
PT_CHG          Inverts the pixel/pixels

*/

#define PT_OFF          0x00
#define PT_ON           0x01
#define PT_CHG          0x02

/* Basic TI Commands

        Function        Description
--------------------------------------------------------------------------
        setxy(x,y)      This funtion moves the text cursor to the speifed
                        (x,y) location

        putm(c)         Puts a charter on the current location

        putc(c)         Puts the charter on the current location and move the
                        cursor over one space

        puts(str)       This command puts a spefied string at the current
                        cursor location and moves the cursor location up

        getkey()        This command waits for a key or the APD register
                        to clear. The calculator will be in low power stand-
                        by during this process. It returns a value refer
                        to the k* table.

*/

setxy(x,y) char x, y; {
#asm
        ld      hl,2
        add     hl,sp
        ld      a,(hl)
        ld      (_curRow),a
        inc     hl
        inc     hl
        ld      a,(hl)
        ld      (_curCol),a
#endasm
}

putm(c) char c; {
#asm
        ld      hl,2
        add     hl,sp
        ld      a,(hl)
        call    _putmap
#endasm
}

putc(c) char c; {
#asm
        ld      hl,2
        add     hl,sp
        ld      a,(hl)
        call    _putc
#endasm
}

puts(str)
char* str;
{
#asm
        ld      hl,2
        add     hl,sp
        ld      a,(hl)
        inc     hl
        ld      h,(hl)
        ld      l,a
; At this point HL is pointer to what we want
;       To add support for \n we will not be using the _puts routine
;       call    _puts
puts_top:
        ld      a, (hl)
        or      a
        jr      z, puts_end
        cp      10
        jr      z, puts_nl
        call    _putc
        inc     hl
        jr      puts_top
puts_nl:
        call    _newline
        inc     hl
        jr      puts_top
puts_end:        
#endasm
}

getkey()
{
#asm
        call    _getkey
        ld      l,a
        xor     a
        ld      h,a
#endasm
}

/* These are the values that would be returned by getkey() */

#define kOn          0x00
#define kRight       0x01
#define kLeft        0x02
#define kUp          0x03
#define kDown        0x04
#define kColon       0x05
#define kEnter       0x06
#define kExit        0x07
#define kClear       0x08
#define kDel         0x09
#define kIns         0x0A
#define kNext        0x0B
#define kAdd         0x0C
#define kSub         0x0D
#define kMul         0x0E
#define kDiv         0x0F
#define kExpon       0x10
#define kLParen      0x11
#define kRParen      0x12
#define kLBrack      0x13
#define kRBrack      0x14
#define kEqual       0x15
#define kStore       0x16
#define kRecall      0x17
#define kComma       0x18
#define kAng         0x19
#define kChs         0x1A
#define kDecPnt      0x1B
#define k0           0x1C
#define k1           0x1D
#define k2           0x1E
#define k3           0x1F
#define k4           0x20
#define k5           0x21
#define k6           0x22
#define k7           0x23
#define k8           0x24
#define k9           0x25
#define kEE          0x26
#define kSpace       0x27
#define kCapA        0x28
#define kCapB        0x29
#define kCapC        0x2A
#define kCapD        0x2B
#define kCapE        0x2C
#define kCapF        0x2D
#define kCapG        0x2E
#define kCapH        0x2F
#define kCapI        0x30
#define kCapJ        0x31
#define kCapK        0x32
#define kCapL        0x33
#define kCapM        0x34
#define kCapN        0x35
#define kCapO        0x36
#define kCapP        0x37
#define kCapQ        0x38
#define kCapR        0x39
#define kCapS        0x3A
#define kCapT        0x3B
#define kCapU        0x3C
#define kCapV        0x3D
#define kCapW        0x3E
#define kCapX        0x3F
#define kCapY        0x40
#define kCapZ        0x41
#define ka           0x42
#define kb           0x43
#define kc           0x44
#define kd           0x45
#define ke           0x46
#define kf           0x47
#define kg           0x48
#define kh           0x49
#define ki           0x4A
#define kj           0x4B
#define kk           0x4C
#define kl           0x4D
#define km           0x4E
#define kn           0x4F
#define ko           0x50
#define kp           0x51
#define kq           0x52
#define kr           0x53
#define ks           0x54
#define kt           0x55
#define ku           0x56
#define kv           0x57
#define kw           0x58
#define kx           0x59
#define ky           0x5A
#define kz           0x5B
#define kVarx        0x5C
#define kAns         0x5D
#define kPi          0x5E
#define kInv         0x5F
#define kSin         0x60
#define kASin        0x61
#define kCos         0x62
#define kACos        0x63
#define kTan         0x64
#define kATan        0x65
#define kSquare      0x66
#define kSqrt        0x67
#define kLn          0x68
#define kExp         0x69
#define kLog         0x6A
#define kALog        0x6B
#define kMath        0x6C
#define kCplx        0x6D
#define kString      0x6E
#define kTest        0x6F
#define kConv        0x70
#define kChar        0x71
#define kBase        0x72
#define kCustom      0x73
#define kVars        0x74
#define kCatalog     0x75
#define kQuit        0x76
#define kLastEnt     0x77
#define kLinkIO      0x78
#define kMem         0x79
#define kList        0x7A
#define kVector      0x7B
#define kConst       0x7C
#define kMatrix      0x7D
#define kPoly        0x7E
#define kSimult      0x7F
#define kStat        0x80
#define kGrMenu      0x81
#define kMode        0x82
#define kPrgm        0x83
#define kCalcu       0x84
#define kSolver      0x85
#define kTable       0x86
#define kBOL         0x87
#define kEOL         0x88
#define kCUSTM       0x93
#define kF1          0xC2
#define kF2          0xC3
#define kF3          0xC4
#define kF4          0xC5
#define kF5          0xC6
#define kF6          0xC7
#define kF7          0xC8
#define kF8          0xC9
#define kF9          0xCA
#define kF10         0xCB
#define kGRAPH       0xFF

#endif /* __TILOADED__ */
